@@ -45,6 +45,8 @@ XML::Filter::XML_Directory_2XHTML - SAX2 filter for munging XML::Directory::SAX
  my $writer = XML::SAX::Writer->new(Output=>$file);
  my $filter = XML::Filter::XML_Directory_2XHTML->new(Handler=>$writer);
 
+ $filter->set_encoding("ISO-8858-1");
+
  # As Canadian as possible, under the circumstances
  $filter->set_lang("en-ca");
 
@@ -105,9 +107,9 @@ use Carp;
 use Exporter;
 use File::Basename;
 
-use XML::Filter::XML_Directory_2::Base '1.4.1';
+use XML::Filter::XML_Directory_2::Base '1.4.4';
 
-$XML::Filter::XML_Directory_2XHTML::VERSION   = '1.2.1';
+$XML::Filter::XML_Directory_2XHTML::VERSION   = '1.3.1';
 @XML::Filter::XML_Directory_2XHTML::ISA       = qw (Exporter XML::Filter::XML_Directory_2::Base);
 @XML::Filter::XML_Directory_2XHTML::EXPORT    = qw();
 @XML::Filter::XML_Directory_2XHTML::EXPORT_OK = qw ();
@@ -208,7 +210,18 @@ The default CSS styles for those classes are :
         margin-bottom:10px;
         }
 
-They can be altered by passing a user-defined CSS stylesheet via the filter's I<set_styles> object method.
+ .thumbnail { display: inline; }
+
+They can be altered by passing a user-defined CSS stylesheet via the filter's I<set_styles> object method. You may also use the I<set_style> method to override the default and assign styles via the HTML <style> element.
+
+Example HTML output:
+
+ <div class = "(file|directory)" id = "...">
+  <div class = "thumbnail">
+   <img src = "..." />
+  </div>
+  <a href = "...">Hello World picture</a>
+ </div>
 
 =head1 OBJECT METHODS
 
@@ -382,6 +395,21 @@ sub set_styles {
   return 1;
 }
 
+=head2 $pkg->set_style(\$css)
+
+You may use this method to override the default styles altogether without also assigning remote stylesheets.
+
+ $pkg->set_style(\qq(.file{ border:2px dotted pink};));
+
+=cut
+
+sub set_style {
+  my $self = shift;
+  if (ref($_[0]) eq "SCALAR") {
+    $self->{__PACKAGE__.'__css'} = $_[0];
+  }
+}
+
 =head2 $pkg->set_scripts(\@scripts)
 
 Define scripts for your document.
@@ -448,7 +476,12 @@ sub _stylesheets {
 					   },
 			      });
 
-  $self->comment({Data=>qq(
+  if ($self->{__PACKAGE__.'__css'}) {
+    $self->comment({Data=>${$self->{__PACKAGE__.'__css'}}});
+  }
+
+  else {
+    $self->comment({Data=>qq(
 .file { 
          border:1px dotted #ccc;
          margin-left:10px;
@@ -461,7 +494,10 @@ sub _stylesheets {
         margin-left:10px;
         margin-bottom:10px;
         }
+ .thumbnail { display:inline; }
+
 )});
+  }
 
   $self->SUPER::end_element({Name=>"style"});
 
@@ -544,7 +580,7 @@ sub _image {
   }
 
   if (ref($src) eq "CODE") {
-    $src = &$src($self->build_uri($data).$self->current_location());
+    $src = &$src($self->build_uri($data)."/".&basename($self->current_location()));
 
     if (ref($src) ne "HASH") { return 0; }
 
@@ -554,6 +590,7 @@ sub _image {
 
   }
 
+  $self->SUPER::start_element({Name=>"div",__PACKAGE__->attributes(class=>"thumbnail")});
   $self->SUPER::start_element({Name=>"img",Attributes=>{
 							"{}src"  => {Name=>"src",
 								     Value=>$src->{'src'},
@@ -577,6 +614,7 @@ sub _image {
 								       NameSpaceURI=>""},
 						       }});
   $self->SUPER::end_element({Name=>"img"});
+  $self->SUPER::end_element({Name=>"div"});
   return 1;
 }
 
@@ -591,11 +629,11 @@ sub _link {
 
   #
 
-  if (my $h = $self->get_handler("linktext")) {
+  if (my $h = $self->retrieve_handler("linktext")) {
     $self->SUPER::characters({Data=>$h->parse_uri($self->build_uri($data))});
   }
   
-  elsif (my $c = $self->get_callback("linktext")) {
+  elsif (my $c = $self->retrieve_callback("linktext")) {
     $self->SUPER::characters({Data=>&$c(
 					$self->build_uri($data),
 					$data->{Attributes}->{'{}name'}->{Value}
@@ -626,10 +664,12 @@ sub start_document {
   my $self = shift;
   $self->SUPER::start_document();
 
-  $self->SUPER::xml_decl({Version=>"1.0"});
-  $self->SUPER::start_dtd({Name=>DTD_HTML_ROOT,
-			   PublicId=>DTD_HTML_PUBLICID,
-			   SystemId=>DTD_HTML_SYSTEMID});
+  $self->SUPER::xml_decl({Version  => "1.0",
+			  Encoding => $self->encoding()});
+
+  $self->SUPER::start_dtd({Name     => DTD_HTML_ROOT,
+			   PublicId => DTD_HTML_PUBLICID,
+			   SystemId => DTD_HTML_SYSTEMID});
   $self->SUPER::end_dtd();
 
   $self->SUPER::start_prefix_mapping({Prefix => "",
@@ -688,11 +728,11 @@ sub start_element {
     $self->_image($type);
     $self->_link($data);
 
-    if (my $h = $self->get_handler($name)) {
+    if (my $h = $self->retrieve_handler($name)) {
       $h->parse_uri($self->build_uri($data));
     }
 
-    elsif (my $c = $self->get_callback($name)) {
+    elsif (my $c = $self->retrieve_callback($name)) {
       $self->SUPER::characters({Data=>&$c($self->build_uri($data))});
     }
 
@@ -716,7 +756,7 @@ sub end_element {
 
     my $title = $self->current_location() || &basename($self->build_uri($data));
 
-    if (my $c = $self->get_callback("title")) {
+    if (my $c = $self->retrieve_callback("title")) {
       $title = &$c();
     }
 
@@ -762,11 +802,11 @@ sub characters {
 
 =head1 VERSION
 
-1.2.1
+1.3.1
 
 =head1 DATE
 
-July 07, 2002
+July 22, 2002
 
 =head1 AUTHOR
 
